import { splitStringValue } from "./utils";
const listenerRegistry = new WeakMap();
export function on(element, type, handler) {
    const list = (Array.isArray(element) ? element : [element]);
    const types = splitStringValue(type);
    for (let i = 0; i < list.length; i++) {
        for (let j = 0; j < types.length; j++) {
            const node = list[i];
            if (null === node) {
                continue;
            }
            const eventType = types[j];
            node.addEventListener(eventType, handler);
            let listeners = listenerRegistry.get(node);
            if (!listeners) {
                listeners = {};
                listenerRegistry.set(node, listeners);
            }
            if (listeners[eventType] === undefined) {
                listeners[eventType] = [];
            }
            listeners[eventType].push(handler);
        }
    }
}
export function off(element, type, handler) {
    const list = (Array.isArray(element) ? element : [element]);
    const types = splitStringValue(type);
    for (let i = 0; i < list.length; i++) {
        for (let j = 0; j < types.length; j++) {
            const node = list[i];
            if (null === node) {
                continue;
            }
            const eventType = types[j];
            node.removeEventListener(eventType, handler);
            const listeners = listenerRegistry.get(node);
            if (listeners !== undefined && listeners[eventType] !== undefined) {
                const index = listeners[eventType].indexOf(handler);
                if (-1 !== index) {
                    listeners[eventType].splice(index, 1);
                }
            }
        }
    }
}
export function onOff(element, type, handler) {
    on(element, type, handler);
    return () => off(element, type, handler);
}
export function once(element, type, handler) {
    if (null === element) {
        return null;
    }
    const intermediate = (event) => {
        handler(event);
        off(element, type, intermediate);
    };
    on(element, type, intermediate);
    return intermediate;
}
export function delegate(element, selector, type, handler) {
    if (null === element) {
        return null;
    }
    const intermediate = (event) => {
        const matchedDelegatedTarget = findDelegatedTarget(element, event.target, selector);
        if (null !== matchedDelegatedTarget) {
            handler(event, matchedDelegatedTarget);
        }
    };
    on(element, type, intermediate);
    return intermediate;
}
function findDelegatedTarget(delegateElement, currentTarget, selector) {
    let node = currentTarget;
    while (null !== node && node !== delegateElement) {
        if (node.matches(selector)) {
            return node;
        }
        node = node.parentElement;
    }
    return null;
}
export function trigger(element, type, data = null) {
    if (null === element) {
        return;
    }
    const event = createEvent(type, {
        bubbles: true,
        cancelable: true,
        detail: data,
    });
    element.dispatchEvent(event);
}
function createEvent(type, args) {
    if (typeof CustomEvent !== "function") {
        const event = document.createEvent('CustomEvent');
        event.initCustomEvent(type, !!args.bubbles, !!args.cancelable, args.detail);
        return event;
    }
    return new CustomEvent(type, args);
}
export function getAllListeners(element) {
    return listenerRegistry.get(element) || {};
}
