import { closest, find, findOne } from "../dom/traverse";
import { delegate, off, on } from "../dom/events";
import { merge } from "../extend";
import mitt from "mitt";
import SortableInteraction from "./Sortable/SortableInteraction";
export default class Sortable {
    constructor(container, config) {
        this.delegateHandler = null;
        this.container = container;
        this.config = merge({
            handle: "",
        }, config);
        this.interaction = null;
        this.emitter = mitt();
        this.listeners = {
            move: this.onDragMove.bind(this),
            end: this.onDragEnd.bind(this),
            mouseOut: this.onMouseOut.bind(this),
            scroll: this.onScroll.bind(this),
        };
    }
    init() {
        if (this.delegateHandler) {
            this.destroy();
        }
        this.delegateHandler = delegate(this.container, `${this.config.items} ${this.config.handle}`, "mousedown", event => this.onInteractionStart(event));
    }
    onInteractionStart(event) {
        if (null !== this.interaction) {
            return;
        }
        let eventTarget = event.target;
        if (null === eventTarget) {
            return;
        }
        const draggedItem = eventTarget.matches(this.config.items) ? eventTarget : closest(eventTarget, this.config.items);
        this.interaction = new SortableInteraction(this.container, draggedItem, this.config.items, event.screenX, event.screenY);
        this.interaction.start();
        on(document.body, "mousemove", this.listeners.move);
        on(document.body, "mouseup", this.listeners.end);
        on(window, "mouseout", this.listeners.mouseOut);
        on(window, "scroll", this.listeners.scroll);
        this.emitter.emit("start", [draggedItem]);
        event.preventDefault();
    }
    onDragMove(event) {
        if (null === this.interaction) {
            return;
        }
        this.interaction.onMove(event.screenX, event.screenY);
    }
    onDragEnd(event) {
        if (null === this.interaction) {
            return;
        }
        off(document.body, "mousemove", this.listeners.move);
        off(document.body, "mouseup", this.listeners.end);
        off(window, "mouseout", this.listeners.mouseOut);
        off(window, "scroll", this.listeners.scroll);
        const endAction = (event !== undefined)
            ? this.interaction.drop(event.screenX, event.screenY)
            : this.interaction.abort();
        endAction
            .then((result) => {
            const currentItems = find(this.config.items, this.container);
            if (null === this.interaction) {
                return;
            }
            const orderHasChanged = this.interaction.orderHasChanged();
            this.interaction = null;
            this.emitter.emit("end");
            if (orderHasChanged) {
                this.emitter.emit("changed", { items: currentItems, result });
            }
        });
    }
    onMouseOut(event) {
        const html = findOne("html");
        if (event.relatedTarget === html) {
            this.onDragEnd();
        }
    }
    onScroll() {
        if (null === this.interaction) {
            return;
        }
        this.interaction.onScroll();
    }
    on(event, callback) {
        this.emitter.on(event, callback);
    }
    destroy() {
        this.onDragEnd();
        if (this.delegateHandler) {
            off(this.container, "mousedown", this.delegateHandler);
            this.delegateHandler = null;
        }
    }
}
